<?php

declare(strict_types=1);

namespace Drupal\a12s_theme_builder\Block;

use Drupal\a12s_core\FormHelperTrait;
use Drupal\block\BlockInterface;
use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformStateInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Template\Attribute;
use Drupal\Core\Url;

/**
 * Provides form for managing menu attributes settings and tools for rendering
 * the menu with their attributes.
 *
 * @todo Integrate with "Menu Attributes" module to support icons fonts.
 */
class MenuBlockAttributes extends SettingsAlterBase {

  use FormHelperTrait;

  /**
   * {@inheritdoc}
   */
  public function defaultValues(): array {
    return [
      'element' => NULL,
      'attribute' => NULL,
      'value' => '',
      'settings' => [
        'levels' => [
          'all' => 1,
          'operator' => 'equal',
          'level' => 1,
        ],
        // @todo handle some conditions.
        //'conditions' => [
        //  'expanded' => '',
        //  'has_children' => '',
        //  'menu_item_uuid' => '',
        //],
        'data_attribute' => [
          'name' => '',
          'json_encode' => FALSE,
        ],
        'aria_attribute' => [
          'name' => '',
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function applies(BlockInterface $entity): bool {
    return str_starts_with($entity->getPluginId(), 'system_menu_block:');
  }

  /**
   * {@inheritdoc}
   */
  public function configurationKey(): string {
    return 'menu_attributes';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array &$form, SubformStateInterface $formState, array $settings = []): void {
    $attributes = $settings['attributes'] ?? [];
    $form += [
      '#type' => 'details',
      '#title' => $this->t('Template attributes'),
      '#open' => !empty($attributes),
      '#parents' => [],
    ];

    // @todo Add a collapsible documentation.

    $form['attributes'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Element'),
        $this->t('Attribute'),
        $this->t('Value'),
        $this->t('Settings'),
      ],
      '#element_validate' => [
        [static::class, 'validateAttributes'],
      ],
      '#prefix' => '<div id="menu-block-attributes-wrapper">',
      '#suffix' => '</div>',
    ];

    $values = $formState->getValue('attributes', []);
    $formParents = $form['#parents'];
    $attributesElement = &$form['attributes'];
    $getCurrentParents = function() use ($formParents, &$attributesElement) {
      return array_merge($formParents, ['attributes', count(Element::children($attributesElement))]);
    };

    foreach ($attributes as $attribute) {
      if (is_array($attribute)) {
        $form['attributes'][] = $this->buildAttributeRow($getCurrentParents(), $attribute);

        if ($values) {
          array_shift($values);
        }
      }
    }

    // Add extra rows.
    foreach ($values as $attribute) {
      if (is_array($attribute)) {
        $form['attributes'][] = $this->buildAttributeRow($getCurrentParents(), $attribute);
      }
    }

    // Add a new row.
    $form['attributes'][] = $this->buildAttributeRow($getCurrentParents());

    $form['add_attributes'] = [
      '#type' => 'button',
      '#value' => $this->t('Add another attribute'),
      '#ajax' => [
        'callback' => [$this, 'addAttributeRow'],
        'wrapper' => 'menu-block-attributes-wrapper',
      ],
    ];
  }

  /**
   * Validates attributes for the full table form element.
   *
   * @param array $element
   *   The form element to validate attributes for.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   * @param array $complete_form
   *   The complete form array.
   *
   * @return void
   */
  public static function validateAttributes(array &$element, FormStateInterface $form_state, array &$complete_form): void {
    $input_exists = FALSE;
    $input = NestedArray::getValue($form_state->getValues(), $element['#parents'], $input_exists);

    if ($input && is_array($input)) {
      foreach ($input as $index => $item) {
        // Is element empty?
        if (empty($item['element']) || empty($item['attribute'])) {
          continue;
        }

        if ($item['attribute'] === 'data') {
          // @todo check for valid data attribute name?
          if (empty($item['settings']['data_attribute']['name'])) {
            $form_state->setError($element[$index]['settings']['data_attribute']['name'], t('The data attribute name is required.'));
          }
        }
        elseif (!(isset($item['value']) && strlen($item['value']))) {
          $form_state->setError($element[$index]['value'], t('The value is required.'));
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, SubformStateInterface $formState): void {
    $values = [];

    foreach ($formState->getValue(['attributes'], []) as $attribute) {
      // Ignore empty values.
      if (empty($attribute['element']) || empty($attribute['attribute'])) {
        continue;
      }

      // Remove default values.
      if ($attribute = $this->cleanValues($attribute)) {
        $values[] = $attribute;
      }
    }

    // Order values.
    usort($values, function($a, $b) {
      $order = ['nav', 'ul', 'li', 'a'];
      $aIndex = array_search($a['element'], $order);
      $bIndex = array_search($b['element'], $order);
      // @todo If equal, order by "all levels" setting?
      return $aIndex - $bIndex;
    });

    $this->saveSettings($values, $formState);
  }

  /**
   * {@inheritdoc}
   */
  protected function saveSettings(array $values, FormStateInterface $formState): void {
    $key = $this->configurationKey();
    /** @var \Drupal\block\Entity\Block $block */
    $block = $formState->getFormObject()->getEntity();
    $thirdPartySetting = $block->getThirdPartySetting('a12s_theme_builder', $key, []);

    // Values are stored under "attributes" key, to allow further integration
    // with menu_attributes module, with icon fonts...
    if (empty($values)) {
      unset($thirdPartySetting['attributes']);
    }
    else {
      $thirdPartySetting['attributes'] = $values;
    }

    parent::saveSettings($thirdPartySetting, $formState);
  }

  /**
   * AJAX callback; return the updated attributes table.
   *
   * @param array &$form
   *   The form array.
   *
   * @return array
   */
  public function addAttributeRow(array &$form): array {
    return $form['a12s_theme_builder']['menu_attributes']['attributes'];
  }

  /**
   * Build the settings for an attribute.
   *
   * @param array $parents
   *   The element parents.
   * @param array $attribute
   *   The attribute values.
   *
   * @return array
   *
   * @todo Add some conditions regarding the "is_expanded", "is_collapsed",
   *   "in_active_trail" conditions or about the link type (link or nolink,
   *   with or without children...).
   */
  protected function buildAttributeRow(array $parents, array $attribute = []): array {
    $this->mergeWithDefault($attribute);

    $row['element'] = [
      '#type' => 'select',
      '#title' => t('Element'),
      '#options' => [
        'nav' => $this->t('Navigation'),
        'ul' => $this->t('List wrapper'),
        'li' => $this->t('List item'),
        'a' => $this->t('Link'),
      ],
      '#empty_option' => $this->t('- Select -'),
      '#title_display' => 'invisible',
      '#default_value' => $attribute['element'],
    ];

    $row ['attribute'] = [
      '#type' => 'select',
      '#title' => t('Attribute'),
      '#options' => [
        'class' => $this->t('CSS Class'),
        'data' => $this->t('Data attribute'),
        'role' => $this->t('Role'),
        'aria' => $this->t('ARIA attribute'),
      ],
      '#empty_option' => $this->t('- Select -'),
      '#title_display' => 'invisible',
      '#default_value' => $attribute['attribute'],
    ];

    // @todo make "value" required when attribute is "class".
    // @todo add AJAX and convert to select for ARIA and role?
    // @todo add placeholder for item title (necessary for aria-label).
    $row['value'] = [
      '#type' => 'textfield',
      '#title' => t('Value'),
      '#title_display' => 'invisible',
      '#default_value' => $attribute['value'],
    ];

    $row['settings'] = [
      '#type' => 'container',
    ];

    $row['settings']['levels'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Levels'),
      '#states' => [
        'invisible' => [
          $this->getInputNameFromPath('select', $parents, 'element') => [
            ['value' => ''],
            ['value' => 'nav'],
          ],
        ],
      ],
    ];

    $row['settings']['levels']['all'] = [
      '#type' => 'checkbox',
      '#title' => t('All levels'),
      '#default_value' => $attribute['settings']['levels']['all'],
    ];

    $row['settings']['levels']['custom'] = [
      '#type' => 'item',
      '#title' => $this->t('Target levels'),
      '#states' => [
        'invisible' => [
          $this->getInputNameFromPath(':input', $parents, ['settings', 'levels', 'all']) => ['checked' => TRUE],
        ],
      ],
      '#wrapper_attributes' => [
        'class' => ['container-inline'],
      ],
    ];

    $row['settings']['levels']['custom']['operator'] = [
      '#type' => 'select',
      '#title' => t('Operator'),
      '#title_display' => 'invisible',
      '#options' => [
        'equal' => $this->t('@label (@operator)', [
          '@label' => $this->t('Equal'),
          '@operator' => '=',
        ]),
        'lower' => $this->t('@label (@operator)', [
          '@label' => $this->t('Lower'),
          '@operator' => '<',
        ]),
        'lower_equal' => $this->t('@label (@operator)', [
          '@label' => $this->t('Lower or equal'),
          '@operator' => '≤',
        ]),
        'greater' => $this->t('@label (@operator)', [
          '@label' => $this->t('Greater'),
          '@operator' => '>',
        ]),
        'greater_equal' => $this->t('@label (@operator)', [
          '@label' => $this->t('Greater or equal'),
          '@operator' => '≥',
        ]),
        'different' => $this->t('@label (@operator)', [
          '@label' => $this->t('Different'),
          '@operator' => '≠',
        ]),
      ],
      '#default_value' => $attribute['settings']['levels']['operator'],
      '#parents' => array_merge($parents, ['settings', 'levels', 'operator']),
    ];

    $row['settings']['levels']['custom']['level'] = [
      '#type' => 'number',
      '#title' => t('Level'),
      '#title_display' => 'invisible',
      '#min' => 1,
      '#default_value' => $attribute['settings']['levels']['level'],
      '#parents' => array_merge($parents, ['settings', 'levels', 'level']),
    ];

    $row['settings']['data_attribute'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Data attribute'),
      '#states' => [
        'visible' => [
          $this->getInputNameFromPath('select', $parents, 'attribute') => ['value' => 'data'],
        ],
      ],
    ];

    $row['settings']['data_attribute']['name'] = [
      '#type' => 'textfield',
      '#title' => t('Name'),
      '#default_value' => $attribute['settings']['data_attribute']['name'],
      '#states' => [
        'required' => [
          $this->getInputNameFromPath('select', $parents, 'attribute') => ['value' => 'data'],
        ],
      ],
    ];

    $row['settings']['data_attribute']['json_encode'] = [
      '#type' => 'checkbox',
      '#title' => t('Encode in JSON format'),
      '#default_value' => $attribute['settings']['data_attribute']['json_encode'],
    ];

    $row['settings']['aria_attribute'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('ARIA attribute'),
      '#states' => [
        'visible' => [
          $this->getInputNameFromPath('select', $parents, 'attribute') => ['value' => 'aria'],
        ],
      ],
    ];

    $row['settings']['aria_attribute']['name'] = [
      '#type' => 'select',
      '#title' => t('Name'),
      '#default_value' => $attribute['settings']['aria_attribute']['name'],
      '#options' => [
        'aria-autocomplete' => $this->t('ARIA autocomplete'),
        'aria-checked' => $this->t('ARIA checked'),
        'aria-disabled' => $this->t('ARIA disabled'),
        'aria-errormessage' => $this->t('ARIA error message'),
        'aria-expanded' => $this->t('ARIA expanded'),
        'aria-haspopup' => $this->t('ARIA has popup'),
        'aria-hidden' => $this->t('ARIA hidden'),
        'aria-invalid' => $this->t('ARIA invalid'),
        'aria-label' => $this->t('ARIA label'),
        'aria-level' => $this->t('ARIA level'),
        'aria-modal' => $this->t('ARIA modal'),
        'aria-multiline' => $this->t('ARIA multiline'),
        'aria-multiselectable' => $this->t('ARIA multiselectable'),
        'aria-orientation' => $this->t('ARIA orientation'),
        'aria-placeholder' => $this->t('ARIA placeholder'),
        'aria-pressed' => $this->t('ARIA pressed'),
        'aria-readonly' => $this->t('ARIA readonly'),
        'aria-selected' => $this->t('ARIA selected'),
        'aria-sort' => $this->t('ARIA sort'),
        'aria-valuemax' => $this->t('ARIA valuemax'),
        'aria-valuemin' => $this->t('ARIA valuemin'),
        'aria-valuenow' => $this->t('ARIA valuenow'),
        'aria-valuetext' => $this->t('ARIA valuetext'),
      ],
      '#states' => [
        'required' => [
          $this->getInputNameFromPath('select', $parents, 'attribute') => ['value' => 'aria'],
        ],
      ],
    ];

    return $row;
  }

  /**
   * {@inheritdoc}
   */
  protected function usePreRenderCallback(): bool {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   *
   * Inject the "menu_attributes" settings to the build array.
   */
  public function preRenderBlock($build): array {
    if ($attributes = &$build['#a12s_theme_builder']['menu_attributes']['attributes']) {
      foreach ($attributes as $attribute) {
        if ($attribute['element'] === 'nav') {
          $build += ['#attributes' => []];
          $this->processAttribute($build['#attributes'], $attribute);
        }
      }

      if (!empty($build['content']['#items'])) {
        $this->processElement($build['content'], $build['content'], [], $attributes, 'ul', 1, '#items');
      }
    }

    return $build;
  }

  /**
   * Apply recursively attributes to a menu and its children.
   *
   * @param array $root
   *   The root element of the menu.
   * @param array $element
   *   The current element being processed.
   * @param array $parents
   *   The full list of parents for the current element.
   * @param array $attributes
   *   An array of attributes to be processed.
   * @param string $type
   *   The type of element being processed. Defaults to 'ul'.
   * @param int $level
   *   The level of the element. Defaults to 1.
   * @param string|null $childKey
   *   The child key for the current element. Defaults to NULL. This is only
   *   necessary as the menu structure is not the same between the first level
   *   and the others.
   */
  protected function processElement(array &$root, array &$element, array $parents, array $attributes, string $type = 'ul', int $level = 1, ?string $childKey = NULL): void {
    // Process the current element.
    foreach ($attributes as $attribute) {
      if ($attribute['element'] !== $type) {
        continue;
      }

      switch ($type) {
        case 'ul':
          // For root UL, things are simple as default template makes use of
          // #attributes.
          if ($level === 1) {
            $element += ['#attributes' => []];
            $this->processAttribute($element['#attributes'], $attribute, $level);
          }
          // But for other UL, we need to find some weird processes... So we
          // store the UL attributes in the parent.
          else {
            $ulAttributes = [];
            $this->processAttribute($ulAttributes, $attribute, $level);

            if (!empty($ulAttributes)) {
              $parentElement = &NestedArray::getValue($root, array_slice($parents, 0, -1));
              $parentElement['ul_attributes'] = new Attribute($ulAttributes);
            }
          }
          break;

        case 'li':
          $element += ['attributes' => new Attribute()];
          $this->processAttribute($element['attributes'], $attribute, $level);
          break;

        case 'a':
          if (isset($element['url']) && $element['url'] instanceof Url) {
            $attributesInstance = $element['url']->getOption('attributes') ?? [];
            $this->processAttribute($attributesInstance, $attribute, $level);
            $element['url']->setOption('attributes', $attributesInstance);
          }
          break;
      }
    }

    // Process children.
    switch ($type) {
      case 'ul':
        if ($childKey) {
          foreach ($element[$childKey] ?? [] as $key => &$item) {
            $this->processElement($root, $item, array_merge($parents, [$childKey, $key]), $attributes, 'li', $level);
          }
        }
        else {
          foreach (Element::children($element) as $key) {
            $this->processElement($root, $element[$key], array_merge($parents, [$key]), $attributes, 'li', $level);
          }
        }
        break;

      case 'li':
        $this->processElement($root, $element, $parents, $attributes, 'a', $level);
        break;

      case 'a':
        if (!empty($element['below'])) {
          $this->processElement($root, $element['below'], array_merge($parents, ['below']), $attributes, 'ul', ($level + 1));
        }
        break;
    }
  }

  /**
   * Processes an attribute and modifies the whole attributes if applicable.
   *
   * @param Attribute|array &$attributes
   *   A reference to the attributes array to modify.
   * @param array $attribute
   *   The attribute to process.
   * @param int $currentLevel
   *   The current level of processing.
   */
  protected function processAttribute(Attribute|array &$attributes, array $attribute, int $currentLevel = 1): void {
    $this->mergeWithDefault($attribute);
    // @todo use DI.
    /** @var \Drupal\a12s_theme_builder\ThemeHelper $themeHelper */
    $themeHelper = \Drupal::service('a12s_theme_builder.helper');

    if ($this->attributeApplies($attribute['settings']['levels'] ?? [], $currentLevel)) {
      switch ($attribute['attribute']) {
        case 'class':
          $value = strtr($attribute['value'] ?? '', ['[level]' => $currentLevel]);
          $themeHelper->addClasses($attributes, $value);
          break;
        case 'data':
          $settings = $attribute['settings']['data_attribute'] ?? [];

          if (!empty($settings['name'])) {
            $name = "data-{$settings['name']}";
            $value = strtr($attribute['value'] ?? '', ['[level]' => $currentLevel]);

            if (!empty($value) && !empty($settings['json_encode'])) {
              $value = Json::encode($value);
            }

            $themeHelper->setAttribute($attributes, $name, $value);
          }
          break;
        case 'role':
          $themeHelper->setAttribute($attributes, 'role', $attribute['value'] ?? '');
          break;
        case 'aria':
          $settings = $attribute['settings']['aria_attribute'] ?? [];
          $value = strtr($attribute['value'] ?? '', ['[level]' => $currentLevel]);

          if (!empty($settings['name'])) {
            $themeHelper->setAttribute($attributes, $settings['name'], $value);
          }
          break;
      }
    }
  }

  /**
   * Determines if the given attribute applies based on the levels settings.
   *
   * @param array $levelsSettings
   *   The settings for different levels:
   *   - 'all' (bool): Indicates if the attribute applies to all levels.
   *   - 'level' (int): The level value to compare with current level.
   *   - 'operator' (string|null): The comparison operator for the level value.
   * @param int $currentLevel
   *   The current level to compare with the level value.
   *
   * @return bool Returns TRUE if the attribute applies, or FALSE otherwise.
   */
  protected function attributeApplies(array $levelsSettings, int $currentLevel): bool {
    if (!empty($levelsSettings['all'])) {
      return TRUE;
    }
    // 0 is not a possible value.
    elseif (!empty($levelsSettings['level'])) {
      $levelValue = $levelsSettings['level'];

      return match ($levelsSettings['operator'] ?? 'unknown') {
        'lower' => $currentLevel < $levelValue,
        'lower_equal' => $currentLevel <= $levelValue,
        'greater' => $currentLevel > $levelValue,
        'greater_equal' => $currentLevel >= $levelValue,
        'different' => $currentLevel != $levelValue,
        'equal' => $currentLevel == $levelValue,
        default => FALSE,
      };
    }

    return FALSE;
  }

}
